/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *   
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *   
 * - Neither name of the copyright holders nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.sf.jmatchparser.util.csv;

import java.io.IOException;
import java.util.Map;

/**
 * Abstract base class to read data from several CSV-like file formats.
 */
public abstract class AbstractCSVWriter {

	/**
	 * Flush this CSV writer.
	 */
	public abstract void flush() throws IOException;

	/**
	 * Close this CSV writer.
	 */
	public abstract void close() throws IOException;

	/**
	 * Write a record (line) into this CSV file.
	 */
	public abstract void write(String... record) throws IOException;

	/**
	 * Write an "indirect" record, consisting of a {@link Map} of headers to
	 * values, and a header list. Values that do not exist in the map are
	 * assumed to be empty.
	 * 
	 * @param recordMap
	 *            map to map headers to values
	 * @param headers
	 *            headers
	 */
	public final <T> void write(Map<T, String> recordMap, T[] headers) throws IOException {
		String[] record = new String[headers.length];
		for (int i = 0; i < headers.length; i++) {
			String value = recordMap.get(headers[i]);
			if (value == null)
				value = "";
			record[i] = value;
		}
		write(record);
	}
}
