/*
 * Copyright (c) 2010 - 2011 Michael Schierl
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *   
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *   
 * - Neither name of the copyright holders nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.sf.jmatchparser.util.csv;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * An {@link AbstractCSVWriter} where you can write records into and then call
 * the {@link #toReader()} method to build an {@link AbstractCSVReader} that can
 * read exactly these records.
 */
public class CSVReaderBuilder extends AbstractCSVWriter {

	private final List<String[]> records = new ArrayList<String[]>();

	@Override
	public void write(String... record) throws IOException {
		records.add(record);
	}

	@Override
	public void flush() throws IOException {
	}

	@Override
	public void close() throws IOException {
	}

	/**
	 * Create a CSV reader from the records written to this builder and reset
	 * it.
	 * 
	 * @see #reset()
	 */
	public AbstractCSVReader toReader() {
		List<String[]> readerRecords = new ArrayList<String[]>(records);
		reset();
		return new CSVReaderBuilderReader(readerRecords.iterator());
	}

	/**
	 * Reset this builder.
	 */
	public void reset() {
		records.clear();
	}

	private static class CSVReaderBuilderReader extends AbstractCSVReader {

		private final Iterator<String[]> it;

		public CSVReaderBuilderReader(Iterator<String[]> it) {
			this.it = it;
		}

		@Override
		public String[] read() throws IOException {
			if (it.hasNext())
				return it.next();
			else
				return null;
		}

		@Override
		public void close() throws IOException {
		}
	}
}
