/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *   
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *   
 * - Neither name of the copyright holders nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.sf.jmatchparser.util.csv;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

/**
 * Class to create CSV files.
 */
public class CSVWriter extends AbstractCSVWriter {
	private final Writer w;
	private final char separator;

	/**
	 * Create a new CSV writer that writes to the given {@link Writer}.
	 * 
	 * @param w
	 *            the writer to write to.
	 */
	public CSVWriter(Writer w, char separator) {
		this.w = w;
		this.separator = separator;
	}

	/**
	 * Create a new CSV writer that writes tot he given stream in the given
	 * charset.
	 * 
	 * @param out
	 *            Output stream
	 * @param charsetName
	 *            charset
	 */
	public CSVWriter(OutputStream out, String charsetName, char separator) throws UnsupportedEncodingException {
		this(new OutputStreamWriter(out, charsetName), separator);
	}

	@Override
	public void flush() throws IOException {
		w.flush();
	}

	@Override
	public void close() throws IOException {
		w.close();
	}

	/**
	 * Get the separator character used by this CSV writer.
	 */
	public char getSeparator() {
		return separator;
	}

	/**
	 * Write a record (line) into this CSV file, properly separating/quoting the
	 * fields as CSV requires.
	 */
	@Override
	public void write(String... record) throws IOException {
		for (int i = 0; i < record.length; i++) {
			if (i != 0)
				w.write(separator);
			w.write("\"" + record[i].replaceAll("\"", "\"\"") + "\"");
		}
		w.write("\r\n");
	}
}
