/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *   
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *   
 * - Neither name of the copyright holders nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.sf.jmatchparser.util.split;

/**
 * A rule where to split a {@link SplitOutputStream}.
 */
public abstract class SplitRule {

	/**
	 * Update the internal state of this split rule after some bytes have been
	 * output.
	 * 
	 * @param data
	 *            bytes that have been output
	 * @param off
	 *            start offset
	 * @param len
	 *            length
	 */
	public abstract void update(byte[] data, int off, int len);

	/**
	 * Check the given bytes whether the output stream has to be split in
	 * between or after the given bytes.
	 * 
	 * @param data
	 *            bytes that should be output
	 * @param off
	 *            start offset
	 * @param len
	 *            length
	 * @return -1 if the given data does not have to be split at all, or the
	 *         length after which the stream has to be split. This value can
	 *         never be zero or greater than <i>len</i>.
	 */
	public abstract int findSplitPos(byte[] data, int off, int len);

	/**
	 * Update the internal state of this split rule after one byte has been
	 * output. The default implementation of this method calls
	 * {@link #update(byte[], int, int)} with a 1-byte byte array.
	 * 
	 * @param data
	 *            byte that has been output
	 */
	public void update(byte data) {
		update(new byte[] { data }, 0, 1);
	}

	/**
	 * Check the given byte whether the output stream has to be split after it.
	 * The default implementation of this method calls
	 * {@link #findSplitPos(byte[], int, int)} with a 1-byte byte array.
	 * 
	 * @param data
	 *            byte that should be output
	 * @return -1 if the given data does not have to be split at all, or 1 if
	 *         the stream has to be split after this byte.
	 */
	public int findSplitPos(byte data) {
		return findSplitPos(new byte[] { data }, 0, 1);
	}
}